@@ -20,12 +20,10 @@
 
 from __future__ import absolute_import, division, print_function
 
-import ast
 import base64
 import contextlib
 import datetime
 import inspect
-import os
 import random as rnd_module
 import traceback
 import warnings
@@ -39,9 +37,9 @@ import hypothesis.strategies as st
 from hypothesis._settings import (
     HealthCheck,
     Phase,
-    PrintSettings,
     Verbosity,
     local_settings,
+    note_deprecation,
     settings as Settings,
 )
 from hypothesis.control import BuildContext
@@ -91,7 +89,7 @@ from hypothesis.internal.reflection import (
 )
 from hypothesis.reporting import current_verbosity, report, verbose_report
 from hypothesis.searchstrategy.collections import TupleStrategy
-from hypothesis.searchstrategy.strategies import SearchStrategy
+from hypothesis.searchstrategy.strategies import MappedSearchStrategy, SearchStrategy
 from hypothesis.statistics import note_engine_for_statistics
 from hypothesis.utils.conventions import infer
 from hypothesis.version import __version__
@@ -214,15 +212,18 @@ def decode_failure(blob):
         )
 
 
-class WithRunner(SearchStrategy):
+class WithRunner(MappedSearchStrategy):
     def __init__(self, base, runner):
         assert runner is not None
-        self.base = base
+        MappedSearchStrategy.__init__(self, base)
         self.runner = runner
 
     def do_draw(self, data):
         data.hypothesis_runner = self.runner
-        return self.base.do_draw(data)
+        return self.mapped_strategy.do_draw(data)
+
+    def __repr__(self):
+        return "WithRunner(%r, runner=%r)" % (self.mapped_strategy, self.runner)
 
 
 def is_invalid_test(name, original_argspec, generator_arguments, generator_kwargs):
@@ -230,6 +231,7 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         def wrapped_test(*arguments, **kwargs):
             raise InvalidArgument(message)
 
+        wrapped_test.is_hypothesis_test = True
         return wrapped_test
 
     if not (generator_arguments or generator_kwargs):
@@ -244,12 +246,11 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         )
 
     if len(generator_arguments) > len(original_argspec.args):
+        args = tuple(generator_arguments)
         return invalid(
-            (
-                "Too many positional arguments for %s() (got %d but"
-                " expected at most %d"
-            )
-            % (name, len(generator_arguments), len(original_argspec.args))
+            "Too many positional arguments for %s() were passed to @given "
+            "- expected at most %d arguments, but got %d %r"
+            % (name, len(original_argspec.args), len(args), args)
         )
 
     if infer in generator_arguments:
@@ -266,16 +267,15 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         if k not in original_argspec.args + original_argspec.kwonlyargs
     ]
     if extra_kwargs and not original_argspec.varkw:
+        arg = extra_kwargs[0]
         return invalid(
-            "%s() got an unexpected keyword argument %r" % (name, extra_kwargs[0])
+            "%s() got an unexpected keyword argument %r, from `%s=%r` in @given"
+            % (name, arg, arg, generator_kwargs[arg])
         )
     for a in original_argspec.args:
         if isinstance(a, list):  # pragma: no cover
             return invalid(
-                (
-                    "Cannot decorate function %s() because it has "
-                    "destructuring arguments"
-                )
+                "Cannot decorate function %s() because it has destructuring arguments"
                 % (name,)
             )
     if original_argspec.defaults or original_argspec.kwonlydefaults:
@@ -284,7 +284,7 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         repr(kw) for kw in original_argspec.kwonlyargs if kw not in generator_kwargs
     ]
     if missing:
-        raise InvalidArgument(
+        return invalid(
             "Missing required kwarg{}: {}".format(
                 "s" if len(missing) > 1 else "", ", ".join(missing)
             )
@@ -424,7 +424,7 @@ def skip_exceptions_to_reraise():
     import unittest
 
     # This is a set because nose may simply re-export unittest.SkipTest
-    exceptions = set([unittest.SkipTest])
+    exceptions = {unittest.SkipTest}
 
     try:  # pragma: no cover
         from unittest2 import SkipTest
@@ -484,11 +484,6 @@ def new_given_argspec(original_argspec, generator_kwargs):
     )
 
 
-ROOT = os.path.dirname(__file__)
-
-STDLIB = os.path.dirname(os.__file__)
-
-
 class StateForActualGivenExecution(object):
     def __init__(self, test_runner, search_strategy, test, settings, random, had_seed):
         self.test_runner = test_runner
@@ -509,14 +504,7 @@ class StateForActualGivenExecution(object):
 
         self.used_examples_from_database = False
 
-    def execute(
-        self,
-        data,
-        print_example=False,
-        is_final=False,
-        expected_failure=None,
-        collect=False,
-    ):
+    def execute(self, data, print_example=False, is_final=False, expected_failure=None):
         text_repr = [None]
         if self.settings.deadline is None:
             test = self.test
@@ -542,8 +530,6 @@ class StateForActualGivenExecution(object):
                 return result
 
         def run(data):
-            if not hasattr(data, "can_reproduce_example_from_repr"):
-                data.can_reproduce_example_from_repr = True
             with local_settings(self.settings):
                 with deterministic_PRNG():
                     with BuildContext(data, is_final=is_final):
@@ -556,10 +542,6 @@ class StateForActualGivenExecution(object):
                                 test.__name__,
                                 arg_string(test, args, kwargs),
                             )
-                            try:
-                                ast.parse(example)
-                            except SyntaxError:
-                                data.can_reproduce_example_from_repr = False
                             report("Falsifying example: %s" % (example,))
                         elif current_verbosity() >= Verbosity.verbose:
                             report(
@@ -724,26 +706,15 @@ class StateForActualGivenExecution(object):
                 # second branch still complains about lack of coverage even if
                 # you add a pragma: no cover to it!
                 # See https://bitbucket.org/ned/coveragepy/issues/623/
-                if self.settings.print_blob is not PrintSettings.NEVER:
-                    failure_blob = encode_failure(falsifying_example.buffer)
-                    # Have to use the example we actually ran, not the original
-                    # falsifying example! Otherwise we won't catch problems
-                    # where the repr of the generated example doesn't parse.
-                    can_use_repr = ran_example.can_reproduce_example_from_repr
-                    if self.settings.print_blob is PrintSettings.ALWAYS or (
-                        self.settings.print_blob is PrintSettings.INFER
-                        and self.settings.verbosity >= Verbosity.normal
-                        and not can_use_repr
-                        and len(failure_blob) < 200
-                    ):
-                        report(
-                            (
-                                "\nYou can reproduce this example by temporarily "
-                                "adding @reproduce_failure(%r, %r) as a decorator "
-                                "on your test case"
-                            )
-                            % (__version__, failure_blob)
+                if self.settings.print_blob:
+                    report(
+                        (
+                            "\nYou can reproduce this example by temporarily "
+                            "adding @reproduce_failure(%r, %r) as a decorator "
+                            "on your test case"
                         )
+                        % (__version__, encode_failure(falsifying_example.buffer))
+                    )
             if self.__was_flaky:
                 flaky += 1
 
@@ -823,10 +794,6 @@ def given(
     """
 
     def run_test_with_generator(test):
-        if hasattr(test, "_hypothesis_internal_test_function_without_warning"):
-            # Pull out the original test function to avoid the warning we
-            # stuck in about using @settings without @given.
-            test = test._hypothesis_internal_test_function_without_warning
         if inspect.isclass(test):
             # Provide a meaningful error to users, instead of exceptions from
             # internals that assume we're dealing with a function.
@@ -979,10 +946,10 @@ def given(
                 if isinstance(runner, TestCase) and hasattr(runner, "subTest"):
                     subTest = runner.subTest
                     try:
-                        setattr(runner, "subTest", fake_subTest)
+                        runner.subTest = fake_subTest
                         state.run()
                     finally:
-                        setattr(runner, "subTest", subTest)
+                        runner.subTest = subTest
                 else:
                     state.run()
             except BaseException as e:
@@ -1055,6 +1022,11 @@ def find(
     # type: (...) -> Any
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
+    note_deprecation(
+        "`find(s, f)` is deprecated, because it is rarely used but takes "
+        "ongoing work to maintain as we upgrade other parts of Hypothesis.",
+        since="2019-07-11",
+    )
     if settings is None:
         settings = Settings(max_examples=2000)
     settings = Settings(settings, suppress_health_check=HealthCheck.all())
